@@ -346,7 +346,11 @@ void qtractorClip::drawClip (
 		rect.setLeft(rect.left() - pSession->pixelFromFrame(iClipOffset) + 1);
 
 	// Draw clip name label...
-	//pPainter->setPen(Qt::white);
+	const QColor& rgbFore
+		= m_pTrack->foreground();
+	pPainter->setPen(m_pTrack->background().value() < 0xcc
+		? rgbFore.lighter(200)
+		: rgbFore.darker(160));
 	pPainter->drawText(rect,
 		Qt::AlignLeft | Qt::AlignBottom | Qt::TextSingleLine, clipTitle());
 
@@ -356,7 +360,7 @@ void qtractorClip::drawClip (
 		return;
 
 	// Fade in/out handle color...
-	QColor rgbFade(m_pTrack->foreground());
+	QColor rgbFade(rgbFore);
 	rgbFade.setAlpha(120);
 	pPainter->setPen(rgbFade);
 	pPainter->setBrush(rgbFade);
