@@ -259,6 +259,10 @@ qtractorPlugin::qtractorPlugin (
 	if (m_pList && m_pType)
 		m_iUniqueID = m_pList->createUniqueID(m_pType);
 
+	// Set instance label...
+	if (m_pType)
+		m_sLabel = m_pType->label();
+
 	// Activate subject properties.
 	m_activateSubject.setName(QObject::tr("Activate"));
 	m_activateSubject.setToggled(true);
@@ -1382,10 +1386,6 @@ void qtractorPlugin::saveCurveFile ( qtractorDocument *pDocument,
 	if (pSession == nullptr)
 		return;
 
-	qtractorOptions *pOptions = qtractorOptions::getInstance();
-	if (pOptions == nullptr)
-		return;
-
 	pCurveFile->clear();
 	pCurveFile->setBaseDir(pSession->sessionDir());
 
@@ -1400,8 +1400,7 @@ void qtractorPlugin::saveCurveFile ( qtractorDocument *pDocument,
 			qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
 			pCurveItem->name = pParam->name();
 			pCurveItem->index = pParam->index();
-			if (pParam->isToggled()	|| pParam->isInteger()
-				|| !pOptions->bSaveCurve14bit) {
+			if (pParam->isToggled()	|| pParam->isInteger()) {
 				const unsigned short controller = (iItem % 0x7f);
 				if (controller == 0x00 || controller == 0x20)
 					++iItem; // Avoid bank-select controllers, please.
@@ -1437,8 +1436,7 @@ void qtractorPlugin::saveCurveFile ( qtractorDocument *pDocument,
 			qtractorCurveFile::Item *pCurveItem = new qtractorCurveFile::Item;
 			pCurveItem->name = pProp->key();
 			pCurveItem->index = pProp->key_index();
-			if (pProp->isToggled()	|| pProp->isInteger()
-				|| !pOptions->bSaveCurve14bit) {
+			if (pProp->isToggled()	|| pProp->isInteger()) {
 				const unsigned short controller = (iItem % 0x7f);
 				if (controller == 0x00 || controller == 0x20)
 					++iItem; // Avoid bank-select controllers, please.
@@ -1581,7 +1579,7 @@ bool qtractorPlugin::savePlugin (
 	pDocument->saveTextElement("index",
 		QString::number(pType->index()), pElement);
 	pDocument->saveTextElement("label",
-		pType->label(), pElement);
+		label(), pElement);
 	pDocument->saveTextElement("preset",
 		preset(), pElement);
 	pDocument->saveTextElement("direct-access-param",
@@ -2322,19 +2320,21 @@ qtractorPlugin *qtractorPluginList::loadPlugin ( QDomElement *pElement )
 
 #if 0
 	if (!sFilename.isEmpty() && !sLabel.isEmpty() &&
-		((pPlugin == nullptr) || ((pPlugin->type())->label() != sLabel))) {
+		((pPlugin == nullptr) || (pPlugin->label() != sLabel))) {
 		iIndex = 0;
 		do {
 			if (pPlugin) delete pPlugin;
 			pPlugin = qtractorPluginFile::createPlugin(this,
 				sFilename, iIndex++, typeHint);
-		} while (pPlugin && (pPlugin->type())->label() != sLabel);
+		} while (pPlugin && (pPlugin->label() != sLabel));
 	}
 #endif
 
 	if (pPlugin) {
 		if (iUniqueID > 0)
 			pPlugin->setUniqueID(iUniqueID);
+		if (!sLabel.isEmpty())
+			pPlugin->setLabel(sLabel);
 		if (iActivateSubjectIndex > 0)
 			pPlugin->setActivateSubjectIndex(iActivateSubjectIndex);
 		pPlugin->setPreset(sPreset);
