@@ -85,6 +85,27 @@ qtractorPluginFactory *qtractorPluginFactory::getInstance (void)
 qtractorPluginFactory::qtractorPluginFactory ( QObject *pParent )
 	: QObject(pParent), m_typeHint(qtractorPluginType::Any)
 {
+	// Register cache file paths...
+	//m_clearFilePaths.clear();
+#ifdef CONFIG_LADSPA
+	addCacheFilePath(qtractorPluginType::Ladspa);
+#endif
+#ifdef CONFIG_DSSI
+	addCacheFilePath(qtractorPluginType::Dssi);
+#endif
+#ifdef CONFIG_VST2
+	addCacheFilePath(qtractorPluginType::Vst2);
+#endif
+#ifdef CONFIG_VST3
+	addCacheFilePath(qtractorPluginType::Vst3);
+#endif
+#ifdef CONFIG_CLAP
+	addCacheFilePath(qtractorPluginType::Clap);
+#endif
+#ifdef CONFIG_LV2
+	addCacheFilePath(qtractorPluginType::Lv2);
+#endif
+
 	// Load persistent blacklist...
 	//m_blacklist.clear();
 	QFile data_file(blacklistDataFilePath());
@@ -118,6 +139,8 @@ qtractorPluginFactory::~qtractorPluginFactory (void)
 	if (data_file.exists())
 		data_file.remove();
 	m_blacklist.clear();
+
+	m_cacheFilePaths.clear();
 }
 
 
@@ -363,6 +386,27 @@ const QStringList& qtractorPluginFactory::blacklist (void) const
 }
 
 
+// Absolute cache file path registration.
+void qtractorPluginFactory::addCacheFilePath (
+	qtractorPluginType::Hint typeHint )
+{
+	const QString& sCacheName = "qtractor_"
+		+ qtractorPluginType::textFromHint(typeHint).toLower()
+		+ "_scan.cache";
+	const QString& sCacheDir
+	#if QT_VERSION < QT_VERSION_CHECK(5, 5, 0)
+		= QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
+	#else
+		= QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
+	#endif
+	const QFileInfo fi(sCacheDir, sCacheName);
+	const QDir& dir = fi.absoluteDir();
+	if (!dir.exists())
+		dir.mkpath(dir.path());
+	m_cacheFilePaths.insert(typeHint, fi.absoluteFilePath());
+}
+
+
 // Absolute temporary blacklist file path.
 QString qtractorPluginFactory::blacklistTempFilePath (void) const
 {
@@ -444,8 +488,7 @@ bool qtractorPluginFactory::startScan ( qtractorPluginType::Hint typeHint )
 	if (pOptions == nullptr)
 		return false;
 
-	bool bDummyPluginScan = pOptions->bDummyPluginScan;
-	int  iDummyPluginHash = 0;
+	int iDummyPluginHash = 0;
 
 	switch (typeHint) {
 	case qtractorPluginType::Ladspa:
@@ -470,43 +513,16 @@ bool qtractorPluginFactory::startScan ( qtractorPluginType::Hint typeHint )
 		break;
 	}
 
-	if (bDummyPluginScan) {
-		const int iNewDummyPluginHash
-			= m_files.value(typeHint).count();
-		Scanner *pScanner = new Scanner(typeHint, this);
-		if (pScanner->open(iDummyPluginHash != iNewDummyPluginHash)) {
-			m_scanners.insert(typeHint, pScanner);
-			switch (typeHint) {
-			case qtractorPluginType::Ladspa:
-				pOptions->iDummyLadspaHash = iNewDummyPluginHash;
-				break;
-			case qtractorPluginType::Dssi:
-				pOptions->iDummyDssiHash = iNewDummyPluginHash;
-				break;
-			case qtractorPluginType::Vst2:
-				pOptions->iDummyVst2Hash = iNewDummyPluginHash;
-				break;
-			case qtractorPluginType::Vst3:
-				pOptions->iDummyVst3Hash = iNewDummyPluginHash;
-				break;
-			case qtractorPluginType::Clap:
-				pOptions->iDummyClapHash = iNewDummyPluginHash;
-				break;
-			case qtractorPluginType::Lv2:
-				pOptions->iDummyLv2Hash = iNewDummyPluginHash;
-				break;
-			default:
-				break;
-			}
-			// Remember to cleanup cache later, when applicable...
-			const QString& sCacheFilePath = pScanner->cacheFilePath();
-			m_cacheFilePaths.insert(typeHint, sCacheFilePath);
-			// Done.
-			return true;
-		}
-	}
+	const QString& sCacheFilePath = m_cacheFilePaths.value(typeHint);
+	if (sCacheFilePath.isEmpty())
+		return false;
 
-	return false;
+	Scanner *pScanner = new Scanner(typeHint, this);
+	if (!pScanner->open(sCacheFilePath, iDummyPluginHash))
+		return false;
+
+	m_scanners.insert(typeHint, pScanner);
+	return true;
 }
 
 
@@ -622,13 +638,43 @@ void qtractorPluginFactory::scan (void)
 
 void qtractorPluginFactory::reset (void)
 {
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions == nullptr)
+		return;
+
 	// Check the proxy (out-of-process) client closure...
 	Scanners::ConstIterator iter = m_scanners.constBegin();
 	const Scanners::ConstIterator& iter_end = m_scanners.constEnd();
 	for ( ; iter != iter_end; ++iter) {
+		const qtractorPluginType::Hint typeHint = iter.key();
 		Scanner *pScanner = iter.value();
-		if (pScanner)
+		if (pScanner) {
 			pScanner->close();
+			const int iDummyPluginHash
+				= pScanner->dummyPluginHash();
+			switch (typeHint) {
+			case qtractorPluginType::Ladspa:
+				pOptions->iDummyLadspaHash = iDummyPluginHash;
+				break;
+			case qtractorPluginType::Dssi:
+				pOptions->iDummyDssiHash = iDummyPluginHash;
+				break;
+			case qtractorPluginType::Vst2:
+				pOptions->iDummyVst2Hash = iDummyPluginHash;
+				break;
+			case qtractorPluginType::Vst3:
+				pOptions->iDummyVst3Hash = iDummyPluginHash;
+				break;
+			case qtractorPluginType::Clap:
+				pOptions->iDummyClapHash = iDummyPluginHash;
+				break;
+			case qtractorPluginType::Lv2:
+				pOptions->iDummyLv2Hash = iDummyPluginHash;
+				break;
+			default:
+				break;
+			}
+		}
 	}
 
 	qDeleteAll(m_scanners);
@@ -843,53 +889,8 @@ bool qtractorPluginFactory::addTypes (
 	Scanner *pScanner = m_scanners.value(typeHint, nullptr);
 	if (pScanner)
 		return pScanner->addTypes(typeHint, sFilename);
-
-#ifdef CONFIG_LV2
-	// Try first URI-based plugin types (LV2...)
-	if (typeHint == qtractorPluginType::Lv2) {
-		qtractorPluginType *pType
-			= qtractorLv2PluginType::createType(sFilename);
-		if (pType == nullptr)
-			return false;
-		if (pType->open()) {
-			addType(pType);
-			pType->close();
-			return true;
-		} else {
-			delete pType;
-			return false;
-		}
-	}
-#endif
-
-	// Try all other file-based types (LADSPA, DSSI, VST...)
-	qtractorPluginFile *pFile = qtractorPluginFile::addFile(sFilename);
-	if (pFile == nullptr)
+	else
 		return false;
-
-	// Add to temporary blacklist...
-	QFile temp_file(blacklistTempFilePath());
-	if (temp_file.exists())
-		readBlacklist(temp_file);
-	writeBlacklist(temp_file, QStringList() << sFilename);
-
-	unsigned long iIndex = 0;
-	while (addTypes(typeHint, pFile, iIndex))
-		++iIndex;
-
-	// If it reaches here safely, then there's
-	// no use to temporary blacklist anymore...
-	temp_file.remove();
-
-	if (iIndex > 0) {
-		pFile->close();
-		return true;
-	}
-
-	// We probably have nothing here.
-	qtractorPluginFile::removeFile(pFile);
-
-	return false;
 }
 
 
@@ -957,7 +958,8 @@ bool qtractorPluginFactory::addTypes (
 // Constructor.
 qtractorPluginFactory::Scanner::Scanner (
 	qtractorPluginType::Hint typeHint, QObject *pParent )
-		: QProcess(pParent), m_typeHint(typeHint), m_iExitStatus(-1), m_bReset(false)
+		: QProcess(pParent), m_typeHint(typeHint),
+			m_iExitStatus(-1), m_iDummyPluginHash(0)
 {
 	QObject::connect(this,
 		SIGNAL(readyReadStandardOutput()),
@@ -972,15 +974,16 @@ qtractorPluginFactory::Scanner::Scanner (
 
 
 // Open/start method.
-bool qtractorPluginFactory::Scanner::open ( bool bReset )
+bool qtractorPluginFactory::Scanner::open (
+	const QString& sCacheFilePath, int iDummyPluginHash )
 {
 	// Cache file setup...
-	m_file.setFileName(cacheFilePath());
+	m_file.setFileName(sCacheFilePath);
 	m_list.clear();
 
-	m_bReset = bReset;
-
 	// Open and read cache file, whether applicable...
+	m_iDummyPluginHash = 0;
+
 	if (m_file.open(QIODevice::ReadOnly | QIODevice::Text)) {
 		// Read from cache...
 		QTextStream sin(&m_file);
@@ -989,27 +992,28 @@ bool qtractorPluginFactory::Scanner::open ( bool bReset )
 			if (sText.isEmpty())
 				continue;
 			const QStringList& props = sText.split('|');
-			if (props.count() >= 6) // get filename...
+			if (props.count() >= 6) { // get filename...
 				m_list[props.at(6)].append(sText);
+				++m_iDummyPluginHash;
+			}
 		}
 		// May close the file.
 		m_file.close();
-	} else {
-		// Force reset...
-		m_bReset = true;
 	}
 
-	if (!m_bReset)
+	if (iDummyPluginHash > 0 && iDummyPluginHash == m_iDummyPluginHash)
 		return true;
 
-	// Make sure cache file location do exists...
-	const QFileInfo fi(m_file);
-	if (!fi.dir().mkpath(fi.absolutePath()))
-		return false;
-
-	// Open cache file for writing...
-	if (!m_file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
-		return false;
+	// Re-open cache file for update...
+	if (!m_file.open(QIODevice::Append | QIODevice::Text)) {
+		// Make sure cache file location do exists...
+		const QFileInfo fi(m_file);
+		if (!fi.dir().mkpath(fi.absolutePath()))
+			return false;
+		// Open cache file for writing...
+		if (!m_file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
+			return false;
+	}
 
 	// LV2 plugins are dang special,
 	// need no out-of-process scanning whatsoever...
@@ -1085,7 +1089,7 @@ void qtractorPluginFactory::Scanner::stdout_slot (void)
 		return;
 
 	const QString sData(QProcess::readAllStandardOutput());
-	addTypes(sData.split('\n'));
+	addTypes(sData.split('\n'), true);
 }
 
 
@@ -1117,10 +1121,12 @@ bool qtractorPluginFactory::Scanner::addTypes (
 		m_typeHint == qtractorPluginType::Lv2 ||
 	#endif
 		QFileInfo(sFilename).exists())) {
-		return addTypes(list);
+		return addTypes(list, false);
 	}
 
-	if (!m_bReset)
+	// If cache file isn't open for update
+	// there's no use to run any further...
+	if (!m_file.isOpen())
 		return false;
 
 	qtractorPluginFactory *pPluginFactory
@@ -1156,6 +1162,7 @@ bool qtractorPluginFactory::Scanner::addTypes (
 				sout << flags.join(",") << '|';
 				sout << sFilename << '|' << 0 << '|';
 				sout << "0x" << QString::number(pType->uniqueID(), 16) << endl;
+				++m_iDummyPluginHash;
 			}
 			// Success.
 			return true;
@@ -1199,7 +1206,8 @@ bool qtractorPluginFactory::Scanner::addTypes (
 }
 
 
-bool qtractorPluginFactory::Scanner::addTypes ( const QStringList& list )
+bool qtractorPluginFactory::Scanner::addTypes (
+	const QStringList& list, bool bDummyPluginType )
 {
 	qtractorPluginFactory *pPluginFactory
 		= static_cast<qtractorPluginFactory *> (QObject::parent());
@@ -1216,8 +1224,10 @@ bool qtractorPluginFactory::Scanner::addTypes ( const QStringList& list )
 			// Brand new type, add to inventory...
 			pPluginFactory->addType(pType);
 			// Cache in...
-			if (m_bReset && m_file.isOpen())
+			if (bDummyPluginType && m_file.isOpen()) {
 				QTextStream(&m_file) << sText << endl;
+				++m_iDummyPluginHash;
+			}
 			// Done.
 		} else {
 			// Possibly some mistake occurred...
@@ -1229,23 +1239,10 @@ bool qtractorPluginFactory::Scanner::addTypes ( const QStringList& list )
 }
 
 
-// Absolute cache file path.
-QString qtractorPluginFactory::Scanner::cacheFilePath (void) const
+// Cache hash result.
+int qtractorPluginFactory::Scanner::dummyPluginHash (void) const
 {
-	const QString& sCacheName = "qtractor_"
-		+ qtractorPluginType::textFromHint(m_typeHint).toLower()
-		+ "_scan.cache";
-	const QString& sCacheDir
-	#if QT_VERSION < QT_VERSION_CHECK(5, 5, 0)
-		= QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
-	#else
-		= QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
-	#endif
-	const QFileInfo fi(sCacheDir, sCacheName);
-	const QDir& dir = fi.absoluteDir();
-	if (!dir.exists())
-		dir.mkpath(dir.path());
-	return fi.absoluteFilePath();
+	return m_iDummyPluginHash;
 }
 
 
