@@ -101,6 +101,9 @@ protected:
 	bool addTypes(qtractorPluginType::Hint typeHint,
 		qtractorPluginFile *pFile, unsigned long iIndex);
 
+	// Absolute cache file path registration.
+	void addCacheFilePath(qtractorPluginType::Hint typeHint);
+
 	// Blacklist file paths.
 	QString blacklistTempFilePath() const;
 	QString blacklistDataFilePath() const;
@@ -163,14 +166,14 @@ public:
 	Scanner(qtractorPluginType::Hint typeHint, QObject *pParent = nullptr);
 
 	// Open/close method.
-	bool open(bool bReset = false);
+	bool open(const QString& sCacheFilePath, int iDummyPluginHash = 0);
 	void close();
 
 	// Service methods.
 	bool addTypes(qtractorPluginType::Hint typeHint, const QString& sFilename);
 
-	// Absolute cache file path.
-	QString cacheFilePath() const;
+	// Cache hash result.
+	int dummyPluginHash() const;
 
 protected slots:
 
@@ -186,7 +189,7 @@ protected:
 	bool start();
 
 	// Service methods (internal)
-	bool addTypes(const QStringList& list);
+	bool addTypes(const QStringList& list, bool bDummyPluginType = false);
 
 private:
 
@@ -202,8 +205,8 @@ private:
 	// Cache hash list.
 	QHash<QString, QStringList> m_list;
 
-	// Cache reset flag.
-	bool m_bReset;
+	// Cache hash result.
+	int m_iDummyPluginHash;
 };
 
 
